<?php

namespace Drupal\views_all_fields\Plugin\views\field;

use Drupal;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;

/**
 * A handler to provide a field that is completely custom by the administrator.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("vaf")
 */
class views_handler_field_views_all_fields extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Pseudo field.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $defaults = parent::defineOptions();
    $defaults['vaf_fields_exclude'] = array('default' => array());
    $defaults['vaf_bundle'] = array('default' => array());
    return $defaults;
  }

  /**
   * {@inheritdoc}
   */
  function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['vaf_bundle'] = array(
      '#title' => t('Bundle'),
      '#description' => t('Pull fields from bundle. Leave blank for all. Resave to see allowed fields.'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#default_value' => $this->options['vaf_bundle'],
    );

    $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($this->definition['entity_type']);

    foreach ($bundles as $key => $bundle_info) {
      $form['vaf_bundle']['#options'][$key] = $bundle_info['label'];
    }

    /* @var $entityFieldManager \Drupal\Core\Entity\EntityFieldManager */
    $entityTypeManager = \Drupal::entityTypeManager();
    /* @var $instances FieldStorageConfig[] */
    $instances = $entityTypeManager->getStorage('field_storage_config')->loadByProperties(['entity_type' => $this->definition['entity_type']]);

    $fields_in_view = $this->view->getHandlers('field');

    foreach ($instances as $field_name => $field) {
      // Drupal chokes on dots in keys.
      $field_name = str_replace('.', '__', $field_name);
      if (!isset($fields_in_view[$field->getName()]) && (empty($this->options['vaf_bundle']) || array_intersect_key($field->getBundles(), $this->options['vaf_bundle']))) {
        $options[$field_name] = "{$field->getLabel()} ({$field->getName()})";
      }
    }

    asort($options);
    $form['vaf_fields_exclude'] = array(
      '#title' => t('Exclude'),
      '#description' => t('Fields marked here will not be pulled. Note that fields already included in this view are already excluded.'),
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $this->options['vaf_fields_exclude'],
    );
  }

}
